@@ -5,16 +5,19 @@ from typing import Optional
 from .ClusterPrintCoreConfiguration import ClusterPrintCoreConfiguration
 
 
-##  Class representing the data of a single slot in the material station.
 class ClusterPrinterMaterialStationSlot(ClusterPrintCoreConfiguration):
-    
-    ## Create a new material station slot object.
-    #  \param slot_index: The index of the slot in the material station (ranging 0 to 5).
-    #  \param compatible: Whether the configuration is compatible with the print core.
-    #  \param material_remaining: How much material is remaining on the spool (between 0 and 1, or -1 for missing data).
-    #  \param material_empty: Whether the material spool is too empty to be used.
+    """Class representing the data of a single slot in the material station."""
+
     def __init__(self, slot_index: int, compatible: bool, material_remaining: float,
                  material_empty: Optional[bool] = False, **kwargs) -> None:
+        """Create a new material station slot object.
+
+        :param slot_index: The index of the slot in the material station (ranging 0 to 5).
+        :param compatible: Whether the configuration is compatible with the print core.
+        :param material_remaining: How much material is remaining on the spool (between 0 and 1, or -1 for missing data).
+        :param material_empty: Whether the material spool is too empty to be used.
+        """
+
         self.slot_index = slot_index
         self.compatible = compatible
         self.material_remaining = material_remaining
