@@ -9,28 +9,34 @@ from UM.Util import parseBool
 from cura.Machines.ContainerNode import ContainerNode
 
 
-##  A QualityGroup represents a group of quality containers that must be applied
-#   to each ContainerStack when it's used.
-#
-#   A concrete example: When there are two extruders and the user selects the
-#   quality type "normal", this quality type must be applied to all stacks in a
-#   machine, although each stack can have different containers. So one global
-#   profile gets put on the global stack and one extruder profile gets put on
-#   each extruder stack. This quality group then contains the following
-#   profiles (for instance):
-#                      GlobalStack       ExtruderStack 1     ExtruderStack 2
-#   quality container: um3_global_normal um3_aa04_pla_normal um3_aa04_abs_normal
-#
-#   The purpose of these quality groups is to group the containers that can be
-#   applied to a configuration, so that when a quality level is selected, the
-#   container can directly be applied to each stack instead of looking them up
-#   again.
 class QualityGroup:
-    ##  Constructs a new group.
-    #   \param name The user-visible name for the group.
-    #   \param quality_type The quality level that each profile in this group
-    #   has.
+    """A QualityGroup represents a group of quality containers that must be applied to each ContainerStack when it's
+    used.
+
+    A concrete example: When there are two extruders and the user selects the quality type "normal", this quality
+    type must be applied to all stacks in a machine, although each stack can have different containers. So one global
+    profile gets put on the global stack and one extruder profile gets put on each extruder stack. This quality group
+    then contains the following profiles (for instance):
+        - GlobalStack
+        - ExtruderStack 1
+        - ExtruderStack 2
+    quality container:
+        - um3_global_normal
+        - um3_aa04_pla_normal
+        - um3_aa04_abs_normal
+
+    The purpose of these quality groups is to group the containers that can be applied to a configuration,
+    so that when a quality level is selected, the container can directly be applied to each stack instead of looking
+    them up again.
+    """
+
     def __init__(self, name: str, quality_type: str) -> None:
+        """Constructs a new group.
+
+        :param name: The user-visible name for the group.
+        :param quality_type: The quality level that each profile in this group has.
+        """
+
         self.name = name
         self.node_for_global = None  # type: Optional[ContainerNode]
         self.nodes_for_extruders = {}  # type: Dict[int, ContainerNode]
